/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_apache_log_line.c,v 1.2 2000/09/15 07:15:55 john Exp $
 */

#include "edir.h"
#include "httpd.h"
#include "http_log.h"

#include "edir_apache_log_line.h"

#ifndef LOG_NOTICE
#define LOG_NOTICE 0
#endif

#ifndef LOG_ERR
#define LOG_ERR 1
#endif

#ifndef LOG_DEBUG
#define LOG_DEBUG 2
#endif

static int use_server_rec = 0;

static server_rec *Srec = NULL;

static request_rec *Rrec = NULL;

static int edir_log_is_daemon = 0;

/**
 * Log a single line of text to the correct log channel.
 * <P>
 * This function should only be called by the general logging functions
 * in 'edir_log.c'.  The common line logging function has been split out
 * into a separate module to allow specific client implementations to
 * integrate logging into the native web server's mechanism.
 *
 * @param level [in]  The logging level, as per syslog.
 * @param line  [in]  Single line of pre-formatted log text.
 */
static void
edir_syslog_line(int level, char *line)
{
#ifdef HAVE_SYSLOG
	if (edir_log_is_daemon)
		syslog(level, "%s", line);
	else
#endif
	switch (level) {
	case LOG_ERR:
		fprintf(stderr, "%s\n", line);
		fflush(stderr);
		break;
	case LOG_DEBUG:
		printf("%s\n", line);
		fflush(stdout);
		break;
	case LOG_NOTICE:
	default:
		printf("%s\n", line);
		break;
	}
}

/**
 * Log a single line of text to the correct log channel.
 * <P>
 * This function should only be called by the general logging functions
 * in 'edir_log.c'.  The common line logging function has been split out
 * into a separate module to allow specific client implementations to
 * integrate logging into the native web server's mechanism.
 *
 * @param level [in]  The logging level, as per syslog.
 * @param line  [in]  Single line of pre-formatted log text.
 */
void edir_log_line(int level, char *line)
{
	if (edir_log_is_daemon) {
		edir_syslog_line(level, line);
		return;
	}
	if ((use_server_rec && (Srec == NULL)) || (Rrec == NULL)) return;
	switch (level) {
	case LOG_ERR:
		if (use_server_rec)
			ap_log_error("EnhydraDirector", 0, APLOG_ERR|APLOG_NOERRNO,
					Srec, "%s", line);
		else
			ap_log_error("EnhydraDirector", 0, APLOG_ERR|APLOG_NOERRNO,
					Rrec->server, "%s", line);
		break;
	case LOG_DEBUG:
		if (use_server_rec)
			ap_log_error("EnhydraDirector", 0, APLOG_DEBUG|APLOG_NOERRNO,
					Srec, "%s", line);
		else
			ap_log_error("EnhydraDirector", 0, APLOG_DEBUG|APLOG_NOERRNO,
					Rrec->server, "%s", line);
		break;
	case LOG_NOTICE:
		if (use_server_rec)
			ap_log_error("EnhydraDirector", 0, APLOG_INFO|APLOG_NOERRNO,
					Srec, "%s", line);
		else
			ap_log_error("EnhydraDirector", 0, APLOG_INFO|APLOG_NOERRNO,
					Rrec->server, "%s", line);
	default:
		printf("%s\n", line);
		break;
	}
}

/**
 * Sets the logging mechanism to use a server_rec.
 *
 * @param rec  [in] Pointer to the server_rec to use.
 */
void edir_apache_log_set_server_rec(server_rec *rec)
{
	use_server_rec = 1;
	Srec = rec;
}

/**
 * Sets the logging mechanism to use a request_rec.
 *
 * @param rec  [in] Pointer to the request_rec to use.
 */
void edir_apache_log_set_request_rec(request_rec *rec)
{
	use_server_rec = 0;
	Rrec = rec;
}

/**
 * Set whether the log system is in 'daemon' mode.
 * <P>
 * In the standard log module, this means "use syslog instead of printf".
 * For apache, it also means 'use syslog', but is only ever called
 * by the child process after forking the Enhydra Director daemon,
 * but before the exec().
 *
 * @param progName [in]  The name of the program doing the logging.
 * @param isDaemon [in]  True for daemon mode, false for "normal".
 */
void edir_log_setdaemon(char *progName, int isDaemon)
{
	if (progName == NULL) progName = "Enhydra Director";
	if (isDaemon) {
		edir_log_is_daemon = 1;
#if defined(HAVE_OPENLOG) && defined(LOG_PID) && defined(LOG_DAEMON)
		closelog();
		openlog(progName, LOG_PID, LOG_DAEMON);
#endif
	} else {
		edir_log_is_daemon = 0;
	}
}

